@@ -11,7 +11,7 @@ def main():
     cf = CloudFlare.CloudFlare(raw=True)
 
     page_number = 0
-    while True: 
+    while True:
         page_number += 1
         try:
             raw_results = cf.zones.get(params={'per_page':5,'page':page_number})
@@ -31,7 +31,7 @@ def main():
         total_count = raw_results['result_info']['total_count']
         total_pages = raw_results['result_info']['total_pages']
 
-        print "COUNT=%d PAGE=%d PER_PAGE=%d TOTAL_COUNT=%d TOTAL_PAGES=%d -- %s" % (count, page, per_page, total_count, total_pages, domains)
+        print("COUNT=%d PAGE=%d PER_PAGE=%d TOTAL_COUNT=%d TOTAL_PAGES=%d -- %s" % (count, page, per_page, total_count, total_pages, domains))
 
         if page_number == total_pages:
             break
